Reviewers: Weiliang,

Message:
PTAL

Description:
X87: [stubs] Properly handle read-only properties in StoreGlobalViaContextStub.

port cac64b9f634743f7f5311d4dca8d50157b10fab5 (r29881)

original commit message:

    We don't need the hole check and slow runtime mode for read-only
    properties this way.

BUG=

Please review this at https://codereview.chromium.org/1263473002/

Base URL: https://chromium.googlesource.com/v8/v8.git@master

Affected files (+8, -12 lines):
  M src/x87/code-stubs-x87.cc


Index: src/x87/code-stubs-x87.cc
diff --git a/src/x87/code-stubs-x87.cc b/src/x87/code-stubs-x87.cc
index 0fe2cd93c5b47f14d387160f27418fc9232f8a72..8b11e028a3a47a7ca38daf2fda3e6abb9334923b 100644
--- a/src/x87/code-stubs-x87.cc
+++ b/src/x87/code-stubs-x87.cc
@@ -4853,23 +4853,14 @@ void StoreGlobalViaContextStub::Generate(MacroAssembler* masm) {
   // Load the PropertyCell at the specified slot.
   __ mov(cell_reg, ContextOperand(context_reg, slot_reg));

-  // Check that cell value is not the_hole.
-  {
-    // TODO(bmeurer): use ecx (name_reg) when name parameter is removed.
-    Register cell_value_reg = cell_details_reg;
- __ mov(cell_value_reg, FieldOperand(cell_reg, PropertyCell::kValueOffset));
-    __ CompareRoot(cell_value_reg, Heap::kTheHoleValueRootIndex);
-    __ j(equal, &slow_case, FLAG_debug_code ? Label::kFar : Label::kNear);
-  }
-
// Load PropertyDetails for the cell (actually only the cell_type and kind).
   __ mov(cell_details_reg,
          FieldOperand(cell_reg, PropertyCell::kDetailsOffset));
   __ SmiUntag(cell_details_reg);
   __ and_(cell_details_reg,
           Immediate(PropertyDetails::PropertyCellTypeField::kMask |
-                    PropertyDetails::KindField::kMask));
-
+                    PropertyDetails::KindField::kMask |
+                    PropertyDetails::kAttributesReadOnlyMask));

   // Check if PropertyCell holds mutable data.
   Label not_mutable_data;
@@ -4895,6 +4886,10 @@ void StoreGlobalViaContextStub::Generate(MacroAssembler* masm) {
   __ cmp(value_reg, FieldOperand(cell_reg, PropertyCell::kValueOffset));
   __ j(not_equal, &not_same_value,
        FLAG_debug_code ? Label::kFar : Label::kNear);
+  // Make sure the PropertyCell is not marked READ_ONLY.
+  __ test(cell_details_reg,
+          Immediate(PropertyDetails::kAttributesReadOnlyMask));
+  __ j(not_zero, &slow_case);
   if (FLAG_debug_code) {
     Label done;
// This can only be true for Constant, ConstantType and Undefined cells, @@ -4919,7 +4914,8 @@ void StoreGlobalViaContextStub::Generate(MacroAssembler* masm) {
   __ Ret();
   __ bind(&not_same_value);

   // Check if PropertyCell contains data with constant type.
   __ cmp(cell_details_reg,
          Immediate(PropertyDetails::PropertyCellTypeField::encode(
                        PropertyCellType::kConstantType) |
@@ -4895,6 +4886,10 @@ void StoreGlobalViaContextStub::Generate(MacroAssembler* masm) {
   __ bind(&not_same_value);

-  // Check if PropertyCell contains data with constant type.
+  // Check if PropertyCell contains data with constant type (and is not
+  // READ_ONLY).
   __ cmp(cell_details_reg,
          Immediate(PropertyDetails::PropertyCellTypeField::encode(


--
--
v8-dev mailing list
v8-dev@googlegroups.com
http://groups.google.com/group/v8-dev
--- You received this message because you are subscribed to the Google Groups "v8-dev" group.
To unsubscribe from this group and stop receiving emails from it, send an email 
to v8-dev+unsubscr...@googlegroups.com.
For more options, visit https://groups.google.com/d/optout.

Reply via email to